// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

#include "manual.h"
#include <stdio.h>

int main() {
    printf(_Generic((ABI_CONSTANT_NAME),
                    char *: "%s",
                    const char *: "%s",
                    char: "%c",
                    signed char: "%hhd",
                    unsigned char: "%hhu",
                    short int: "%hd",
                    unsigned short int: "%hu",
                    int: "%d",
                    unsigned int: "%u",
                    long: "%ld",
                    unsigned long: "%lu",
                    long long: "%lld",
                    unsigned long long: "%llu",
                    double: "%f",
                    long double: "%ld"),
           ABI_CONSTANT_NAME);
    return 0;
}
