// This file was generated by gir (https://github.com/gtk-rs/gir)
// from gir-files (https://github.com/gtk-rs/gir-files)
// DO NOT EDIT

use Font;
use Fontset;
use Language;
use ffi;
use glib::object::IsA;
use glib::translate::*;
use glib_ffi;
use gobject_ffi;
use std::mem;
use std::ptr;

glib_wrapper! {
    pub struct FontsetSimple(Object<ffi::PangoFontsetSimple, ffi::PangoFontsetSimpleClass>): Fontset;

    match fn {
        get_type => || ffi::pango_fontset_simple_get_type(),
    }
}

impl FontsetSimple {
    pub fn new(language: &mut Language) -> FontsetSimple {
        unsafe {
            from_glib_full(ffi::pango_fontset_simple_new(language.to_glib_none_mut().0))
        }
    }
}

pub trait FontsetSimpleExt {
    fn append(&self, font: &Font);

    fn size(&self) -> i32;
}

impl<O: IsA<FontsetSimple>> FontsetSimpleExt for O {
    fn append(&self, font: &Font) {
        unsafe {
            ffi::pango_fontset_simple_append(self.to_glib_none().0, font.to_glib_none().0);
        }
    }

    fn size(&self) -> i32 {
        unsafe {
            ffi::pango_fontset_simple_size(self.to_glib_none().0)
        }
    }
}
